transform = {}

-- Bidimensional vector --
function vector2d(x, y)

    local vect = {}
    
    vect.x = x
    vect.y = y

    return vect
end

-- 4 directions vector --
function dualvect(lt, rt, up, dw)

    local dual = {}

    dual.lt = lt
    dual.rt = rt
    dual.up = up
    dual.dw = dw

    return dual
end

function transform:load()

    local other = {}

    setmetatable(other, self)
    self.__index = self

    return other
end

function transform:init(fpos, ctrl)
    
    -- Current position --
    self.cpos = fpos

    -- Input control --
    self.ctrl = ctrl
    self.side = 1

    -- Animations --
    self.state = 'idle'

-- # Phisics control ------------------------------------- # --

    -- Screen collision --
    self.scale = love.graphics.getWidth() / 160
    self.d_col = dualvect(0, (love.graphics.getWidth() / self.scale) - 8, 0, (love.graphics.getHeight() / self.scale) - 8)
    self.w_col = sys.dutable(self.d_col)

    -- Movement --
    self.x_speed = 1.2
    self.y_speed = 0

    -- Jump --
    self.jmp_cnt = true
    self.jmp_stg = -240
    self.gravity = self.jmp_stg * (self.jmp_stg / (self.jmp_stg / 3.2))
    self.crt_gty = self.gravity
end

function transform:move(dt)

    -- Reset collision paramters --
    self.w_col = sys.dutable(self.d_col)
    platform.col(self.cpos, self.w_col, love.keyboard.isDown(self.ctrl.dwn))

-- # Default control ------------------------------------- # --
    
    if type(self.ctrl.lft) == "string" then

        -- Go to left --
        if love.keyboard.isDown(self.ctrl.lft) and self.cpos.x - self.x_speed <= self.w_col.rt then
            
            self.cpos.x = self.cpos.x - self.x_speed
            self.side = -1

            self.state = 'moving'

        -- Go to right --
        elseif love.keyboard.isDown(self.ctrl.rgt) and self.cpos.x + self.x_speed >= self.w_col.lt then

            self.cpos.x = self.cpos.x + self.x_speed
            self.side = 1

            self.state = 'moving'
        
        -- Not moving --
        else self.state = 'idle' end

-- # By system ctrl -------------------------------------- # --

    elseif self.ctrl ~= nil then

        -- Go to left --
        if self.ctrl.lft == true and self.cpos.x - self.x_speed <= self.w_col.rt then

            self.cpos.x = self.cpos.x - 1
            self.side = -1
        end

        -- Go to right --
        if self.ctrl.rgt == true and self.cpos.x + self.x_speed >= self.w_col.lt then

            self.cpos.x = self.cpos.x + 1
            self.side = 1
        end
    end

-- # Gravity control ------------------------------------- # --

    -- Gravity --
    if self.y_speed ~= 0 or self.cpos.y ~= self.w_col.dw then

        self.cpos.y = self.cpos.y + self.y_speed * dt
        self.y_speed = self.y_speed - self.gravity * dt
    end

    -- On Landing and on roof --
    if self.cpos.y >= self.w_col.dw or self.cpos.y <= self.w_col.up then

        self.y_speed = 0

        if self.cpos.y >= self.w_col.dw then
            
            self.jmp_cnt = true
            self.cpos.y = self.w_col.dw
        end
    end

-- # Collision ------------------------------------------- # --
    
    -- Fix wall trhu --
    transform:fixp(self.cpos, self.w_col)
end

function transform:btnp(key)
    
    -- Jump if on floor --
    if key == self.ctrl.ups and self.jmp_cnt then

        self.y_speed = (self.jmp_stg / 2) * 1.5
        self.jmp_cnt = false
    end
end

function transform:fixp(vect2d, limit)

    -- Keep char in window --
    while vect2d.x < limit.lt do
        
        vect2d.x = vect2d.x + 0.1
    end

    while vect2d.x > limit.rt do
        
        vect2d.x = vect2d.x - 0.1
    end

    while vect2d.y < limit.up do
        
        vect2d.y = vect2d.y + 0.1
    end

    while vect2d.y > limit.dw do
        
        vect2d.y = vect2d.y - 0.1
    end
end

function transform:getp()

    transform:fixp(self.cpos, self.w_col)
    return self.cpos
end

function transform:setp(npos)

    self.cpos = npos
    transform:fixp(self.cpos, self.w_col)
end

function transform:stat()

    return self.state
end

function transform:jump()

    return self.y_speed
end

function transform:gets()

    return self.side
end

function transform:draw()

    love.graphics.rectangle("fill", self.cpos.x, self.cpos.y, 8, 8)
end