-- Internal system resources --
require("Libs/internals_lib")
require("Libs/save_load_lib")

-- Physics resources --
require("Libs/transform_lib")
require("Libs/collision_lib")

-- Graphic resources --
require("Libs/resources_lib")
require("Libs/urenderer_lib")
require("Libs/tiledmaps_lib")

-- Running time data --
local data = {['door'] = 1}
local room = require("rooms/room_3")

-- Effects settings --
local work = true
local wait = false
local timr = 0

-- Viewpoint --
local offset = vector2d(0, 0)

-- Global tilemap dictionary --
dict = {

    -- Grass and dirt block --
    ['G'] = {['x'] = 00, ['y'] = 07},
    ['D'] = {['x'] = 01, ['y'] = 07},

    -- Backgorund dirt block --
    ['g'] = {['x'] = 02, ['y'] = 07},
    ['d'] = {['x'] = 03, ['y'] = 07},

    -- Grass --
    ['0'] = {['x'] = 00, ['y'] = 06},
    ['1'] = {['x'] = 01, ['y'] = 06},
    ['2'] = {['x'] = 02, ['y'] = 06},
    ['3'] = {['x'] = 03, ['y'] = 06},

    -- Wood --
    ['W'] = {['x'] = 04, ['y'] = 07},
    ['P'] = {['x'] = 04, ['y'] = 06},
    ['w'] = {['x'] = 05, ['y'] = 07},
    ['p'] = {['x'] = 05, ['y'] = 06},

    -- Marbel blocks --
    ['O'] = {['x'] = 09, ['y'] = 00},
    ['<'] = {['x'] = 10, ['y'] = 00},
    ['>'] = {['x'] = 11, ['y'] = 00},
    ['A'] = {['x'] = 09, ['y'] = 01},
    ['V'] = {['x'] = 09, ['y'] = 02},
    ['H'] = {['x'] = 12, ['y'] = 00},

    -- Marbel variant --
    ['o'] = {['x'] = 12, ['y'] = 02},
    ['h'] = {['x'] = 12, ['y'] = 01},

    -- Other marbel blocks --
    ['L'] = {['x'] = 10, ['y'] = 01},
    ['J'] = {['x'] = 11, ['y'] = 02},
    ['l'] = {['x'] = 10, ['y'] = 01},
    ['j'] = {['x'] = 11, ['y'] = 02},

    ['M'] = {['x'] = 13, ['y'] = 02},
    ['='] = {['x'] = 13, ['y'] = 00},
    ['_'] = {['x'] = 13, ['y'] = 01},

    -- vineyards --
    ['?'] = {['x'] = 14, ['y'] = 00},
    [':'] = {['x'] = 14, ['y'] = 01},
    [';'] = {['x'] = 14, ['y'] = 02},

    ['T'] = {['x'] = 15, ['y'] = 00},
    ['|'] = {['x'] = 15, ['y'] = 01},
    ['/'] = {['x'] = 15, ['y'] = 02},

    -- Sword --
    [']'] = {['x'] = 14, ['y'] = 03},
    ['['] = {['x'] = 15, ['y'] = 03},
    ['E'] = {['x'] = 14, ['y'] = 04},
    ['F'] = {['x'] = 15, ['y'] = 04},

    -- Statues --
    ['n'] = {['x'] = 09, ['y'] = 03},
    ['N'] = {['x'] = 09, ['y'] = 04},

    ['r'] = {['x'] = 10, ['y'] = 03},
    ['R'] = {['x'] = 10, ['y'] = 04},

    ['s'] = {['x'] = 11, ['y'] = 03},
    ['S'] = {['x'] = 11, ['y'] = 04},

    ['u'] = {['x'] = 12, ['y'] = 03},
    ['U'] = {['x'] = 12, ['y'] = 04},

    ['c'] = {['x'] = 13, ['y'] = 03},
    ['C'] = {['x'] = 13, ['y'] = 04},

    -- Empty --
    ['.'] = {['x'] = 15, ['y'] = 15},
}

thru = {'p', 'P', '='}
skip = {
    
    -- Background --
    '.', '_',
    'g', 'd',
    'w',
    
    -- Sword --
    ']', '[',
    
    -- Statues --
    's', 'S', 'c', 'C'
}

-- Character controller --
ctrl = {
    
    ['lft'] = 'left',
    ['rgt'] = 'right',
    ['dwn'] = 'down',
    
    ['int'] = 'up',
    ['ups'] = 'x',
    ['atk'] = 'c',
}

function love.load()

    -- Set window configuration --
    love.window.setMode(480, 480, {resizable = false, vsync = true, minwidth = 80, minheight = 80})
    wdth, hght = love.graphics.getWidth() / 3, love.graphics.getWidth() / 3

    -- Fix resolution --
    love.graphics.setDefaultFilter("nearest")

    -- Text Font --
    font = love.graphics.newFont("Etthernal.ttf", 16)
    love.graphics.setFont(font)

    -- Load graphic resources --
    sres.load('sprite_sheet', 'tiled_map')

--#---------------------------------------------------------#--
    
    memory.load(data)

    random = 0
    math.randomseed(love.timer.getTime())

    -- Init room --
    room.load()
end

function love.update(dt)

    -- Set random frame --
    random = math.random(0, 32)

    -- Next room --
    if room.out == true then

        -- Pause running time --
        wait = true

        -- Start timer --
        timr = timr + 200 * dt 

        -- End of fade --
		if timr > 255 then

            timr = 0
            wait = false

            -- Load last room --
            if room.rid == 0 then 
                
                room = require('rooms/room_' .. tostring(data.door))
                room.load()

            else
                
                data[data.door] = true
                data.door = data.door + 1

                room = require('rooms/room_' .. tostring(data.door))
                room.load()
            end
        
            memory.save(data)
        end
    end

    -- If room isn't nil and it's not waiting --
    if room and not wait and work then room.update(dt) end
end

function love.keypressed(key)
    
    -- Close program --
    if key == "escape" then love.event.quit() end

    -- Pause --
    if key == "return" and data.door ~= 0 then work = not work end

    if key == "r" then room.load() end

    -- Call key press function of room --
    if room and not wait then room.keypressed(key) end
end

function love.draw()

    -- Fix window scale --
    love.graphics.scale(love.graphics.getWidth() / 160, love.graphics.getWidth() / 160)

    -- Set viewpoint --
    love.graphics.translate(offset.x, offset.y)

    -- If room isn't nil --
    if room then room.draw() end

--# System stuff -------------------------------------------#--

    -- If game is paused --
    if not work then
        
        local r, g, b, a = love.graphics.getColor()
        
		love.graphics.setColor(0, 0, 0, 0.5)
        love.graphics.rectangle("fill", 0, 0, 400, 400)
        
        love.graphics.setColor(1, 1, 1, 1)
        love.graphics.print("Pause", (wdth / 2) - 30, (hght / 2) - 8)

		love.graphics.setColor(r, g, b, a)
    end

    -- Transition --
    if wait then

        local r, g, b, a = love.graphics.getColor()
        
        love.graphics.setColor(1, 1, 1, timr / 255)
        love.graphics.rectangle("fill", 0, 0, wdth, hght)
        love.graphics.setColor(r, g, b, a)
    end
end