require("Libs/draw_text_lib")
require("Libs/ghost_int_lib")

room = {}

room.rid = 2
room.out = false

-- Fade stuff --
local open = false
local blnk = false
local wait = false
local timr = 0

local skey = false

-- Text drawning --
local text_I, text_II, text_III

function room.load()

    -- Background Color --
    love.graphics.setBackgroundColor(181/255, 80/255, 136/255, 1)

-- # Character ------------------------------------------- # --

    -- Init character --
    char = transform:load()
    char:init(vector2d(0, 144), ctrl)

    -- Character animator --
    anim = rend:load()
    anim:init()
    
    -- Character animations --
    idle = {[0] = 0, [1] = 1, [2] = 2, [3] = 3}
    walk = {[0] = 0, [1] = 4, [2] = 3, [3] = 5}

-- # Sword ----------------------------------------------- # --

    -- Sword --
    atkng = false
    ended = false

    sword = rend:load()
    sword:init(vector2d(52, 128))

    atack = {[0] = 8, [1] = 9, [2] = 10, [3] = 11, [4] = 12, [5] = 12, [6] = 12, [7] = 12}

-- # Level Design ---------------------------------------- # --

    -- Set Map --
    base, deco = love.filesystem.load("rooms/room_2.txt")()
    map.tmap(base, deco, dict, thru, skip)
    
    -- Reset fade --
    open = false
    blnk = false
        
    timr = 0
    wait = false
end

function room.update(dt)
-- # Movement and animations ----------------------------- # --

    char:move(dt)

    -- Jumping --
    if char:jump() < 0 then anim:play(dt, 0, {[0] = 6}, 8, true)
    elseif char:jump() > 0 then anim:play(dt, 0, {[0] = 7}, 8, true)

    -- Walking and idle --
    elseif char:stat() == 'moving' then anim:play(dt, 0, walk, 8, true)
    else anim:play(dt, 0, idle, 8, true) end

    -- Sword --
    if ended == true then
        
        atkng = false
        ended = false
    end

    -- Check if sword's animation ended --
    if atkng then ended = sword:play(dt, 0, atack, 16) end

-- # Text ------------------------------------------------ # --

    -- Create text I --
    if not text_I and not text_II then

        text_I = textbox.type("Go away,go!", 8, 10, {7, 10}, 1)
    
    else
        
        -- Update text II --
        if text_I then textbox.write(dt, text_I) end

        -- Replace by text II --
        if not text_II then
            
            -- Create text II --
            if text_I.ended == true then

                text_I = nil
                text_II = textbox.type("There is", 8, 10)
            end

        else

            -- Update text II --
            textbox.write(dt, text_II)
            
            -- Create text III --
            if text_II.ended == true then
            
                if not text_III then text_III = textbox.type("nothing here", 8, 22)
                else textbox.write(dt, text_III) end
            end
        end
    end

-- # Next room --------------------------------------------# --

    if math.floor(char:getp().x + 0.5) >= wdth - 8 and text_III.ended == true then room.out = true end

-- # Fade out -------------------------------------------- # --

    if not open then

        wait = true
        timr = timr + 200 * dt 
            
        if timr > 255 then
            
            timr = 255
            wait = false
            open = true
        end
    end
end

function room.keypressed(key)

    -- Update button press of char --
    char:btnp(key)

    -- Sword atack --
    if key == ctrl.atk and skey == true then
        
        sword:loop()
        atkng = true
    end
end

function room.draw()

    -- Draw tilemap --
    map.draw()

    -- Draw character --
    anim:draw(char:getp(), char:gets())

    -- Sword --
    if atkng then

        if char:gets() > 0 then sword:draw(vector2d(char:getp().x + 8, char:getp().y), char:gets()) end
        if char:gets() < 0 then sword:draw(vector2d(char:getp().x - 8, char:getp().y), char:gets()) end
    end

    -- Draw text --
    textbox.draw({text_I, text_II, text_III}, 'noises')

-- # Level transition ------------------------------------ # --

    -- Transition --
    if wait then

        local r, g, b, a = love.graphics.getColor()
        
        love.graphics.setColor(1, 1, 1, (255 - timr) / 255)
        love.graphics.rectangle("fill", 0, 0, 400, 400)
		love.graphics.setColor(r, g, b, a)
    end

    -- Fill blank space --
    if not blnk then
        
        local r, g, b, a = love.graphics.getColor()
        
        love.graphics.setColor(1, 1, 1, 1)
        love.graphics.rectangle("fill", 0, 0, 160, 160)
		love.graphics.setColor(r, g, b, a)
        
        love.timer.sleep(0.1)

        blnk = true
    end
end

return room